'use strict';

var CustomerMgr = require('dw/customer/CustomerMgr');
var Site = require('dw/system/Site');
var Transaction = require('dw/system/Transaction');
var Logger = require('dw/system/Logger');
var CustomerMapping = require('~/cartridge/scripts/lib/mapping/customer');
var helper = require('~/cartridge/scripts/lib/mapping/helper/helper');
var FileWriter = require('dw/io/FileWriter');
var File = require('dw/io/File');
var CSVStreamWriter = require('dw/io/CSVStreamWriter');
var cordial = require('~/cartridge/scripts/services/сordial');

/**
 * 
 * This function export customers
 */
function exportCustomers(options) {
  var searchProfiles;

  var cordialCustomersDateUpdate = 
    Site.getCurrent().getCustomPreferenceValue('cordialCustomersDateUpdate');

  var newCordialCustomersDateUpdate = helper.getTime();
  var errorFlag = false;

  // TODO cordialCustomersDateUpdate
  Logger.error(['start-exportCustomers', cordialCustomersDateUpdate]);

  if (cordialCustomersDateUpdate) {
    searchProfiles = CustomerMgr.searchProfiles(
      'lastModified >= {0}',
      null, 
      cordialCustomersDateUpdate);
  } else {
    var date = newCordialCustomersDateUpdate;
    searchProfiles = CustomerMgr.searchProfiles('creationDate <= {0}', null, date);
  }

  while (searchProfiles.hasNext()) {
    var contact = searchProfiles.next();

    if (contact.custom.cordialCID) {
      cordial.UpdateContacts(contact, function () {}, function (response) {
        errorFlag = true;
        Logger.error('Import customer error {0} on update', response);
        Logger.error(['exportCustomers--contact-data', contact.email, contact.custom.cordialCID, JSON.stringify(contact)]);
      });
    } else {
      cordial.GetContacts({
        'email': contact.getCredentials().getLogin()
      }, function (response) {
        var cordialContact = JSON.parse(response.getObject());
        var cID = cordialContact[0].attributes.cID;
        var Transaction = require('dw/system/Transaction');
        Transaction.wrap(function () {
          contact.custom.cordialCID = cID;
        });
        cordial.UpdateContacts(contact, function () {}, function (response) {
          errorFlag = true;
          Logger.error('Import customer error {0} on update', response);
        });
      }, function () {
        cordial.PostContacts(contact, function () {}, function (response) {
          errorFlag = true;
          Logger.error('Import customer error {0} on post', response);
        });
      });
    }
  }

  if (!errorFlag) {
    Transaction.wrap(function () {
      Site.getCurrent().setCustomPreferenceValue('cordialCustomersDateUpdate', newCordialCustomersDateUpdate);
    })
  }
}


/**
 * 
 * Test Run: export customers file
 **/
function exportCustomersFileTestRun(options) {
  var searchProfiles;
  var newCordialCustomersDateUpdate = helper.getTime();

  var date = helper.getTime();
  Logger.error(['call-exportCustomersFileTestRun', date, 1000]);
  searchProfiles = CustomerMgr.searchProfiles('creationDate <= {0}', null, date);

  exportCustomersFileRunner(searchProfiles, options, 1000);
}

/**
 * 
 * This function export customers files
 **/
function exportCustomersFile(options) {
  var searchProfiles;
  var newCordialCustomersDateUpdate = helper.getTime();

  var date = helper.getTime();
  searchProfiles = CustomerMgr.searchProfiles('creationDate <= {0}', null, date);

  exportCustomersFileRunner(searchProfiles, options, 0);

  Transaction.wrap(function () {
    Site.getCurrent().setCustomPreferenceValue('cordialCustomersDateUpdate', newCordialCustomersDateUpdate);
  })
}

function exportCustomersFileRunner(searchProfiles, options, limit) {
  var counter = 0;

  if (!limit) {
    limit = 999999999;
  }

  if (searchProfiles.hasNext()) {
    //Create CSV
    var siteID = Site.getCurrent().getID(),
      exportFolderPath = File.IMPEX + "/cordial/",
      exportFolder = new File(exportFolderPath);

    if (!exportFolder.exists()) {
      exportFolder.mkdirs();
    }

    var exportFilename = exportFolder.fullPath + File.SEPARATOR + "customers_" + siteID + ".csv";

    var exportFile = new File(exportFilename),
        fileWriter = new FileWriter(exportFile, "utf-8"),
        writer = new CSVStreamWriter(fileWriter, ',', '"'),
        isHeaders = false,
        columnNames = [];
    Logger.error(['call-exportCustomersFile', exportFile]);

    while (searchProfiles.hasNext() && (counter++ < limit)) {
      var contact = searchProfiles.next();
      var customerMapping = new CustomerMapping();
      var json = Site.getCurrent().getCustomPreferenceValue('cordialCustomerMapping');
      var newJson = json.replace(/([a-zA-Z0-9]+?):/g, '"$1":');
      newJson = newJson.replace(/'/g, '"');
      var cordialCustomerMapping = JSON.parse(newJson);
      var cordialCustomer = customerMapping.execute(contact, cordialCustomerMapping);
      var flattenCordialCustomer = helper.flattenObject(cordialCustomer);

      // Write CSV header
      if (!isHeaders) {
        //columnNames = Object.keys(flattenCordialCustomer);
        columnNames = [];
        cordialCustomerMapping.forEach(function (mSpecial) {
          columnNames.push(mSpecial.cordial);
        });

        Logger.error(['exportCustomersFile--headers', columnNames, contact.email, typeof flattenCordialCustomer.birthday, newJson]);
        writer.writeNext(columnNames);
        isHeaders = true;
      }

      //Write customer info
      var values = [];

      columnNames.forEach(function(name) {
        if (flattenCordialCustomer[name] === undefined) {
          //values.push('undefined');
          values.push('');
        } else {
          if (typeof flattenCordialCustomer[name] === 'boolean') {
            values.push(flattenCordialCustomer[name] ? 1 : 0);
          } else if (name === 'birthday' || name === 'nextbirthday') {
            values.push(flattenCordialCustomer[name].toISOString().split('T')[0]);
          } else {
            values.push(flattenCordialCustomer[name]);
          }
        }
      });

      writer.writeNext(values);
    }

    //Clear resources
    writer.close();
    fileWriter.close();
    Logger.error(['call-exportCustomersFile', exportFilename, limit, counter]);

    cordial.PostJobCustomers(
      exportFile.fullPath, columnNames, options, function () {}, 
      function (response) {
        Logger.error('Post job error {0}', response);
      });
  }
}


module.exports = {
  exportCustomers: exportCustomers,
  exportCustomersFile: exportCustomersFile,
  exportCustomersFileTestRun: exportCustomersFileTestRun
};
