'use strict';

var OrderMgr = require('dw/order/OrderMgr');
var Site = require('dw/system/Site');
var Transaction = require('dw/system/Transaction');
var Logger = require('dw/system/Logger');
var OrderMapping = require('int_cordial_core/cartridge/scripts/lib/mapping/order');
var FileWriter = require('dw/io/FileWriter');
var File = require('dw/io/File');
var cordial = require('~/cartridge/scripts/services/сordial');
var helper = require('~/cartridge/scripts/lib/mapping/helper/helper');

/**
 * exportOrders
 * This function export orders
 */
function exportOrders(options) {
  var searchOrders;
  var cordialOrdersDateUpdate = Site.getCurrent().getCustomPreferenceValue('cordialOrdersDateUpdate');
  var newCordialOrdersDateUpdate = helper.getTime();
  var errorFlag = false;


  // TODO cordialOrdersDateUpdate
  if (cordialOrdersDateUpdate) {
    searchOrders = OrderMgr.queryOrders('lastModified >= {0}', null, cordialOrdersDateUpdate);
  } else {
    var date = newCordialOrdersDateUpdate;
    searchOrders = OrderMgr.queryOrders('creationDate <= {0}', null, date);
  }
  while (searchOrders.hasNext()) {
    var order = searchOrders.next();
    cordial.GetOrders(order.getOrderNo(), function (response) {
      var oldOrder = JSON.parse(response.object);
      cordial.PostOrders(order, function () {}, function (response) {
        errorFlag = true;
        Logger.error('Import order error {0} on post', response);
      }, oldOrder);
    }, function () {
      cordial.PostOrders(order, function () {}, function (r) {
          errorFlag = true;
          Logger.error('Import order error {0} on post', r);
      });
    });

  }

  if (!errorFlag) {
    Transaction.wrap(function () {
      Site.getCurrent().setCustomPreferenceValue('cordialOrdersDateUpdate', newCordialOrdersDateUpdate);
    })
  }
}

/**
 * 
 * This function export orders files
 */
function exportOrdersFile(options) {
  var searchOrders;
  var newCordialOrdersDateUpdate = helper.getTime();

  var date = newCordialOrdersDateUpdate;
  searchOrders = OrderMgr.queryOrders('creationDate <= {0}', null, date);

  exportOrdersFileRuner(options, searchOrders, 999999999);

  Transaction.wrap(function () {
    Site.getCurrent().setCustomPreferenceValue('cordialOrdersDateUpdate', newCordialOrdersDateUpdate);
  })
}

function exportOrdersFileTestRun(options) {
  var searchOrders;
  var newCordialOrdersDateUpdate = helper.getTime();

  var date = newCordialOrdersDateUpdate;
  searchOrders = OrderMgr.queryOrders('creationDate <= {0}', null, date);

  exportOrdersFileRuner(options, searchOrders, 100);
}

function exportOrdersFileRuner(options, searchOrders, limit) {
  var newCordialOrdersDateUpdate = helper.getTime();
  var date = newCordialOrdersDateUpdate;
  var counter = 0;

  if (searchOrders.hasNext()) {
    //Create jsonl
    var siteID = Site.getCurrent().getID(),
      exportFolderPath = File.IMPEX + "/cordial/",
      exportFolder = new File(exportFolderPath);

    if (!exportFolder.exists()) {
      exportFolder.mkdirs();
    }

    var exportFilename = exportFolder.fullPath + File.SEPARATOR + "orders_" + siteID + ".jsonl";
    var exportFile = new File(exportFilename),
      fileWriter = new FileWriter(exportFile, "utf-8");

    while (searchOrders.hasNext() && (counter++ < limit)) {
      var orders = searchOrders.next();
      var orderMapping = new OrderMapping();
      var cordialOrder = orderMapping.execute(orders);

      fileWriter.writeLine(JSON.stringify(cordialOrder));
    }

    //Clear resources
    fileWriter.close();

    Logger.error(['call-exportOrdersFile', newCordialOrdersDateUpdate, exportFilename, limit, counter]);

    cordial.PostJobOrders(exportFile.fullPath, options, 
      function (response) {
        Logger.error(['cordial.PostJobOrders-response', response]);
      }, 
      function (response) {
        Logger.error('Post job error {0}', response);
      });
  }
  
}

module.exports = {
  exportOrders: exportOrders,
  exportOrdersFileTestRun: exportOrdersFileTestRun,
  exportOrdersFile: exportOrdersFile
};
