'use strict';

var ProductMgr = require('dw/catalog/ProductMgr');
var ProductInventory = require('dw/catalog/ProductInventoryMgr');

var Site = require('dw/system/Site');
var ProductMapping = require('int_cordial_core/cartridge/scripts/lib/mapping/product');
var Transaction = require('dw/system/Transaction');
var Logger = require('dw/system/Logger');
var FileWriter = require('dw/io/FileWriter');
var File = require('dw/io/File');
var cordial = require('~/cartridge/scripts/services/сordial');
var helper = require('~/cartridge/scripts/lib/mapping/helper/helper');

/**
 * exportProducts
 * This function export products
 */
function exportProducts(options) {
  var searchProducts;

  var searchInventory = ProductInventory.getInventoryList();

  var cordialProductsDateUpdate = new Date(
    Site.getCurrent().getCustomPreferenceValue('cordialProductsDateUpdate')
  );
  cordialProductsDateUpdate.setDate(cordialProductsDateUpdate.getDate() - 2);

  var newCordialProductsDateUpdate = helper.getTime();
  var errorFlag = false;

  searchProducts = ProductMgr.queryAllSiteProductsSorted();
  Logger.error(['Call-exportProducts', cordialProductsDateUpdate]);

  while (searchProducts.hasNext()) {
    var product = searchProducts.next();

    if (cordialProductsDateUpdate) {
      var productUpdatedRecently = product.lastModified >= cordialProductsDateUpdate;

      // Check inventory recent changes
      var inventoryDetails = searchInventory.getRecord(product);

      if (inventoryDetails) 
      if (inventoryDetails.getAllocationResetDate() >= cordialProductsDateUpdate) {
        Logger.error(['product-update-inventoryDetails', product, inventoryDetails]);
        productUpdatedRecently = true;
      }

      if (productUpdatedRecently) {
        Logger.error(['exportProducts-cordial.PostProducts', product, product.constructor.name]);

        cordial.PostProducts(
          product,
          function () {}, 
          function (response) { 
            errorFlag = true; 
            Logger.error('Import product error {0} on post', response); 
          }
        );

        if (product.constructor.name === 'dw.catalog.Variant') {
          cordial.PostProducts(
            product.getMasterProduct(),
            function () {}, 
            function (response) { 
              errorFlag = true; 
              Logger.error('Import product error {0} on post', response); 
            }
          );
        }
      }
    } else {
      cordial.PostProducts(product, function () {}, function (response) {
        errorFlag = true;
        Logger.error('Import product error {0} on post', response);
      });
    }
  }

  if (!errorFlag) {
    Transaction.wrap(function () {
      Site.getCurrent().setCustomPreferenceValue('cordialProductsDateUpdate', newCordialProductsDateUpdate);
    });
  }
}

/**
 * 
 * This function export products files
 */
function exportProductsFile(options) {
  var searchProducts;
  var newCordialProductsDateUpdate = helper.getTime();

  searchProducts = ProductMgr.queryAllSiteProductsSorted();

  if (searchProducts.hasNext()) {
    // Create CSV for export
    var siteID = Site.getCurrent().getID(),
      exportFolderPath = File.IMPEX + "/cordial/",
      exportFolder = new File(exportFolderPath);

    if (!exportFolder.exists()) {
      exportFolder.mkdirs();
    }

    var exportFilename = exportFolder.fullPath + File.SEPARATOR + "products_" + siteID + ".jsonl";
    var exportFile = new File(exportFilename);
      fileWriter = new FileWriter(exportFile, "utf-8");
    Logger.error(['Call-exportProductsFile', searchProducts.count, exportFilename]);

    while (searchProducts.hasNext()) {
      var product = searchProducts.next();
      var productMapping = new ProductMapping();
      var cordialProduct = productMapping.execute(product)

      fileWriter.writeLine(JSON.stringify(cordialProduct));
    }

    fileWriter.close();

    cordial.PostJobProducts(exportFile.fullPath, options, function () {}, function (response) {
      Logger.error('Post job error {0}', response);
    });
  }

  Transaction.wrap(function () {
    Site.getCurrent().setCustomPreferenceValue('cordialProductsDateUpdate', newCordialProductsDateUpdate);
  });
}

module.exports = {
  exportProducts: exportProducts,
  exportProductsFile: exportProductsFile
};
