'use strict';
var System = require('dw/system/System');
var Calendar = require('dw/util/Calendar');

function get(obj, path, defaultValue) {
  return path.split(".").reduce(function (a, c) {
    return a && a[c] ? a[c] : defaultValue || null;
  }, obj);
};

function set(obj, path, value) {
  if (Object(obj) !== obj) return obj; // When obj is not an object
  // If not yet an array, get the keys from the string-path
  if (!Array.isArray(path)) path = path.toString().match(/[^.[\]]+/g) || [];
  path.slice(0, -1).reduce(function (a, c, i) {
    return (// Iterate all of them except the last one
      Object(a[c]) === a[c] // Does the key exist and is its value an object?
      // Yes: then follow that path
      ? a[c]
      // No: create the key. Is the next key a potential array-index?
      : a[c] = Math.abs(path[i + 1]) >> 0 === +path[i + 1] ? [] // Yes: assign a new array object
      : {}
    );
  }, // No: assign a new plain object
  obj)[path[path.length - 1]] = value; // Finally assign the value to the last key
  return obj; // Return the top-level object to allow chaining
};

function flattenObject(ob) {
  var toReturn = {};

  for (var i in ob) {
      if (!ob.hasOwnProperty(i)) continue;
      toReturn[i] = ob[i];

      if ((typeof ob[i]) == 'object' && ob[i] !== null) {
          var flatObject = flattenObject(ob[i]);
          for (var x in flatObject) {
              if (!flatObject.hasOwnProperty(x)) continue;

              toReturn[i + '.' + x] = flatObject[x];
          }
      } else {
          toReturn[i] = ob[i];
      }
  }
  return toReturn;
}

function getTime() {
  var tzString =  System.getInstanceTimeZone();
  var calendar = new Calendar();

  calendar.setTimeZone(tzString);
  return calendar.getTime();
}
  
  module.exports = {
    get: get,
    set: set,
    flattenObject: flattenObject,
    getTime: getTime
  };
